"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TwitterLink = void 0;
const react_1 = __importDefault(require("react"));
const react_2 = require("@chakra-ui/react");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const react_3 = require("@strata-foundation/react");
const spl_utils_1 = require("@strata-foundation/spl-utils");
const useLinkTwitter_1 = require("../hooks/useLinkTwitter");
const useQueryString_1 = require("../hooks/useQueryString");
const react_4 = require("react");
const react_async_hook_1 = require("react-async-hook");
const fa_1 = require("react-icons/fa");
const WUMBO_IDENTITY_SERVICE_URL = process.env.NEXT_PUBLIC_WUMBO_IDENTITY_SERVICE_URL || "http://localhost:8082";
const link = (publicKey, state, code, ownerTwitterHandle, provider) => __awaiter(void 0, void 0, void 0, function* () {
    if (!ownerTwitterHandle) {
        if (state == localStorage.getItem("state") && provider && publicKey) {
            yield (0, spl_utils_1.executeRemoteTxn)(provider, WUMBO_IDENTITY_SERVICE_URL + "/twitter/oauth", {
                pubkey: publicKey.toBase58(),
                redirectUri: localStorage.getItem("redirectUri"),
                code,
                state,
            });
            localStorage.removeItem("state");
            localStorage.removeItem("redirectUri");
            return true;
        }
    }
});
const TwitterLink = () => {
    const { publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const [code, setCode] = (0, useQueryString_1.useQueryString)("code", "");
    const { provider, awaitingApproval } = (0, react_3.useProvider)();
    const [state, setState] = (0, useQueryString_1.useQueryString)("state", "");
    const { nameString: ownerTwitterHandle, error: reverseTwitterError } = (0, react_3.useReverseName)(publicKey || undefined, react_3.WUMBO_TWITTER_VERIFIER, react_3.WUMBO_TWITTER_TLD);
    const { handleErrors } = (0, react_3.useErrorHandler)();
    const { execute } = (0, useLinkTwitter_1.useLinkTwitter)();
    const { loading: linkLoading, error: linkError, result, } = (0, react_async_hook_1.useAsync)(link, [publicKey, state, code, ownerTwitterHandle, provider]);
    handleErrors(reverseTwitterError, linkError);
    (0, react_4.useEffect)(() => {
        if (result) {
            setCode("");
            setState("");
        }
    }, [result, setCode, setState]);
    if (publicKey) {
        return (react_1.default.createElement(react_2.Button, { colorScheme: "twitter", isDisabled: !!ownerTwitterHandle, leftIcon: react_1.default.createElement(react_2.Icon, { as: fa_1.FaTwitter }), variant: ownerTwitterHandle ? "ghost" : "outline", onClick: execute, isLoading: linkLoading, loadingText: awaitingApproval ? "Awaiting Approval" : "Linking" }, ownerTwitterHandle ? ownerTwitterHandle : "Link Wallet"));
    }
    return null;
};
exports.TwitterLink = TwitterLink;
//# sourceMappingURL=TwitterLink.js.map