var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Button, Icon } from "@chakra-ui/react";
import { useWallet } from "@solana/wallet-adapter-react";
import { useErrorHandler, useProvider, useReverseName, WUMBO_TWITTER_TLD, WUMBO_TWITTER_VERIFIER } from "@strata-foundation/react";
import { executeRemoteTxn } from "@strata-foundation/spl-utils";
import { useLinkTwitter } from "../hooks/useLinkTwitter";
import { useQueryString } from "../hooks/useQueryString";
import { useEffect } from "react";
import { useAsync } from "react-async-hook";
import { FaTwitter } from "react-icons/fa";
const WUMBO_IDENTITY_SERVICE_URL = process.env.NEXT_PUBLIC_WUMBO_IDENTITY_SERVICE_URL || "http://localhost:8082";
const link = (publicKey, state, code, ownerTwitterHandle, provider) => __awaiter(void 0, void 0, void 0, function* () {
    if (!ownerTwitterHandle) {
        if (state == localStorage.getItem("state") && provider && publicKey) {
            yield executeRemoteTxn(provider, WUMBO_IDENTITY_SERVICE_URL + "/twitter/oauth", {
                pubkey: publicKey.toBase58(),
                redirectUri: localStorage.getItem("redirectUri"),
                code,
                state
            });
            localStorage.removeItem("state");
            localStorage.removeItem("redirectUri");
            return true;
        }
    }
});
export const TwitterLink = () => {
    const { publicKey } = useWallet();
    const [code, setCode] = useQueryString("code", "");
    const { provider, awaitingApproval } = useProvider();
    const [state, setState] = useQueryString("state", "");
    const { nameString: ownerTwitterHandle, error: reverseTwitterError } = useReverseName(publicKey || undefined, WUMBO_TWITTER_VERIFIER, WUMBO_TWITTER_TLD);
    const { handleErrors } = useErrorHandler();
    const { execute } = useLinkTwitter();
    const { loading: linkLoading, error: linkError, result } = useAsync(link, [
        publicKey,
        state,
        code,
        ownerTwitterHandle,
        provider,
    ]);
    handleErrors(reverseTwitterError, linkError);
    useEffect(() => {
        if (result) {
            setCode("");
            setState("");
        }
    }, [result, setCode, setState]);
    if (publicKey) {
        return (<Button colorScheme="twitter" isDisabled={!!ownerTwitterHandle} leftIcon={<Icon as={FaTwitter}/>} variant={ownerTwitterHandle ? "ghost" : "outline"} onClick={execute} isLoading={linkLoading} loadingText={awaitingApproval ? "Awaiting Approval" : "Linking"}>
        {ownerTwitterHandle ? ownerTwitterHandle : "Link Wallet"}
      </Button>);
    }
    return null;
};
