import React from "react";
import { SimpleGrid, HStack, Text, Spinner } from "@chakra-ui/react";
import { Creator, usePublicKey, useTokenBondingFromMint, useTokenMetadata } from "@strata-foundation/react";
const onCreatorClick = (c, t, tokenRef, handle) => {
    window.open(tokenRef
        ? `https://wum.bo/app/profile/view/${tokenRef.mint}`
        : handle
            ? `https://twitter.com/${handle}`
            : `https://explorer.solana.com/address/${c.toBase58()}`, "_blank");
};
const InfoItem = ({ name, creator, loading }) => {
    return (<HStack spacing={2}>
      <Text fontWeight={800} color="gray.700">
        {name}:{" "}
      </Text>{" "}
      {loading ? (<Spinner size="xs"/>) : creator ? (<Creator creator={creator} onClick={onCreatorClick}/>) : "None"}
    </HStack>);
};
export const AuthorityAndTokenInfo = ({ mintKey }) => {
    const { info: tokenBonding, loading } = useTokenBondingFromMint(mintKey);
    const { metadata, loading: loadingMeta } = useTokenMetadata(mintKey);
    const updateAuthority = usePublicKey(metadata === null || metadata === void 0 ? void 0 : metadata.updateAuthority);
    return (<SimpleGrid columns={[1, 1, 2]} fontSize="14px" spacing={4}>
      <InfoItem creator={updateAuthority} loading={loading} name="Requester"/>
      <InfoItem creator={tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.reserveAuthority} loading={loading} name="Approver"/>
    </SimpleGrid>);
};
