var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { HStack, Spinner, StackDivider, VStack, Text } from "@chakra-ui/react";
import { useConnection } from "@solana/wallet-adapter-react";
import { Creator, useTokenAccount, useMint, roundToDecimals } from "@strata-foundation/react";
import { toNumber } from "@strata-foundation/spl-token-bonding";
import { useAsync } from "react-async-hook";
import { numberWithCommas } from "../../utils/numberWithCommas";
const onCreatorClick = (c, t, tokenRef, handle) => {
    window.open(tokenRef
        ? `https://wum.bo/app/profile/view/${tokenRef.mint.toBase58()}`
        : handle
            ? `https://twitter.com/${handle}`
            : `https://explorer.solana.com/address/${c.toBase58()}`, "_blank");
};
function getLargestTokenAccounts(connection, mint) {
    return __awaiter(this, void 0, void 0, function* () {
        if (mint) {
            const accounts = (yield connection.getTokenLargestAccounts(mint)).value;
            return accounts
                .filter((a) => a.uiAmount)
                .map((account) => ({
                publicKey: account.address,
                amount: account.uiAmount,
            }));
        }
        return [];
    });
}
export const TopHolders = ({ mintKey }) => {
    const { connection } = useConnection();
    const { result: accounts, loading } = useAsync(getLargestTokenAccounts, [
        connection,
        mintKey,
    ]);
    if (loading) {
        return <Spinner />;
    }
    return <VStack w="full" spacing={2} divider={<StackDivider borderColor="gray.200"/>}>
    {accounts === null || accounts === void 0 ? void 0 : accounts.map((account, index) => (<LeaderboardItem rank={index + 1} key={account.publicKey.toBase58()} {...account}/>))}
  </VStack>;
};
const LeaderboardItem = ({ publicKey, rank }) => {
    const { info: account, loading } = useTokenAccount(publicKey);
    const mint = useMint(account === null || account === void 0 ? void 0 : account.mint);
    if (loading || !account) {
        return <Spinner />;
    }
    return (<HStack align="center" w="full" padding={2} justify="space-between">
      <HStack align="center" spacing={8}>
        <Text fontWeight={500} color="gray.500">
          {rank}
        </Text>
        <Creator creator={account.owner} onClick={onCreatorClick}/>
      </HStack>
      <Text fontWeight={700} color="gray.600" marginLeft="auto">
        {mint &&
            numberWithCommas(roundToDecimals(toNumber(account.amount, mint), mint.decimals))}
      </Text>
    </HStack>);
};
