var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Alert, Button, Center, Heading, Image, Input, Spinner, Text, VStack } from "@chakra-ui/react";
import { useWallet } from "@solana/wallet-adapter-react";
import { Notification, useBondingPricing, useMint, useOwnedAmount, useProvider, useStrataSdks, useTokenBondingFromMint, useTokenMetadata } from "@strata-foundation/react";
import React, { useState } from "react";
import { useAsyncCallback } from "react-async-hook";
import toast from "react-hot-toast";
import { SaleItemAdmin } from "./SaleItemAdmin";
function buy(tokenBondingSdk, tokenBonding, quantity) {
    return __awaiter(this, void 0, void 0, function* () {
        yield tokenBondingSdk.buy({
            tokenBonding,
            desiredTargetAmount: quantity,
            slippage: 0.05
        });
        toast.custom((t) => (<Notification show={t.visible} type="success" heading="Transactoin Successful" message={`Successfully purchased ${quantity}`} onDismiss={() => toast.dismiss(t.id)}/>));
    });
}
export const SaleItem = ({ mintKey, name, description, image, }) => {
    var _a, _b;
    const { info: tokenBonding, loading: bondingLoading } = useTokenBondingFromMint(mintKey);
    const { connected, publicKey } = useWallet();
    const targetMint = useMint(tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.targetMint);
    const { image: targetImage, metadata: targetMetadata, data: targetData, loading: targetMetaLoading, } = useTokenMetadata(tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.targetMint);
    const { metadata: baseMetadata, loading: metadataLoading } = useTokenMetadata(tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.baseMint);
    const { pricing, loading: pricingLoading } = useBondingPricing(tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey);
    const { tokenBondingSdk } = useStrataSdks();
    const [qty, setQty] = useState("0");
    const { execute, loading, error } = useAsyncCallback(buy);
    const { awaitingApproval } = useProvider();
    const qtyNumber = Number(qty);
    const mintCapNumber = (_a = tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.mintCap) === null || _a === void 0 ? void 0 : _a.toNumber();
    const targetSupplyNumber = targetMint === null || targetMint === void 0 ? void 0 : targetMint.supply.toNumber();
    const balance = useOwnedAmount(tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.baseMint);
    const price = pricing === null || pricing === void 0 ? void 0 : pricing.buyTargetAmount(Number(qty) || 1);
    const notEnoughFunds = (balance || 0) < (price || 0);
    const passedMintCap = (targetSupplyNumber || 0) >= (mintCapNumber || 0);
    const isAdmin = publicKey &&
        ((_b = tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.reserveAuthority) === null || _b === void 0 ? void 0 : _b.equals(publicKey));
    name = (targetMetadata === null || targetMetadata === void 0 ? void 0 : targetMetadata.data.name) || name;
    image = targetImage || image;
    description = (targetData === null || targetData === void 0 ? void 0 : targetData.description) || description;
    const dataMissing = !name && !image && !description;
    if (!targetMetaLoading && dataMissing) {
        return (<Center height="300px">
        <Text>
          <b>404: </b> Not found
        </Text>
      </Center>);
    }
    if (dataMissing) {
        return <Spinner />;
    }
    return (<VStack p={4} align="start">
      {isAdmin && tokenBonding && (<SaleItemAdmin tokenBondingKey={tokenBonding.publicKey}/>)}
      <Heading>{name}</Heading>
      <Image alt={name} w="full" src={image}/>
      <Text whiteSpace="pre-line">{description}</Text>
      <div id="price">
        <Input placeholder="Quantity" onChange={(event) => setQty(event.target.value)} type="number" min={1}/>
        <Text size="sm">
          <b>Price:</b> {pricingLoading ? <Spinner size="xs"/> : price}{" "}
          {baseMetadata === null || baseMetadata === void 0 ? void 0 : baseMetadata.data.symbol}
        </Text>
        <Text size="sm">
          <b>Available:</b>{" "}
          {bondingLoading ? (<Spinner size="xs"/>) : (<span>
              {(mintCapNumber || 0) - (targetSupplyNumber || 0)} /{" "}
              {mintCapNumber || 0}
            </span>)}
        </Text>
      </div>
      {error && (<Alert status="error">
          <Alert status="error">{error.toString()}</Alert>
        </Alert>)}
      <Button isDisabled={!connected ||
            notEnoughFunds ||
            passedMintCap ||
            !qtyNumber ||
            qtyNumber <= 0} isLoading={loading} value={qty} loadingText={awaitingApproval ? "Awaiting Approval" : "Loading"} onClick={() => qtyNumber &&
            qtyNumber > 0 &&
            execute(tokenBondingSdk, tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey, qtyNumber)} w="full" size="lg" colorScheme="blue">
        {!connected
            ? "Connect a Wallet"
            : notEnoughFunds
                ? `Not Enough ${(baseMetadata === null || baseMetadata === void 0 ? void 0 : baseMetadata.data.symbol) || "funds"}`
                : passedMintCap
                    ? "Sold Out"
                    : "Buy"}
      </Button>
    </VStack>);
};
