var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { MarketplaceSdk } from "@strata-foundation/marketplace-sdk";
import { useProvider } from "@strata-foundation/react";
import React, { useContext, useMemo } from "react";
import { useAsync } from "react-async-hook";
export const MarketplaceSdkContext = React.createContext({
    loading: true,
});
function tryProm(prom) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield prom;
        }
        catch (e) {
            console.error(e);
        }
        return undefined;
    });
}
function getSdk(provider) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!provider) {
            return undefined;
        }
        return tryProm(MarketplaceSdk.init(provider));
    });
}
export const MarketplaceSdkProviderRaw = ({ children }) => {
    const { provider } = useProvider();
    const { result, loading, error } = useAsync(getSdk, [provider]);
    const sdks = useMemo(() => ({
        marketplaceSdk: result,
        error,
        loading,
    }), [result, loading, error]);
    return (<MarketplaceSdkContext.Provider value={sdks}>
      {children}
    </MarketplaceSdkContext.Provider>);
};
export const MarketplaceSdkProvider = ({ children }) => {
    return (<MarketplaceSdkProviderRaw>{children}</MarketplaceSdkProviderRaw>);
};
export const useMarketplaceSdk = () => {
    return useContext(MarketplaceSdkContext);
};
